% Generates polynomial features of specified degree for training set in order
% to make decision boundaries to have complex form.
% This function is used to extend training set features with new features to get 
% more complex form/shape of decision boundaries.

function out = add_polynomial_features(X1, X2, degree)
  % Returns a new feature array with more features, comprising of 
  % X1, X2, X1.^2, X2.^2, X1*X2, X1*X2.^2, etc..
	
  % a = zeros(N) creates a NxN matrix 
  % Create a Nx1 matrix where N = size of vector X1
  out = ones(size(X1(:, 1)))
  for i = 1:degree
   for j = 0:i
    % Add the polynomial at the last column of the unit matrix
	% if y =   [
	% 1  1  1
	% 1  1  1
	% 1  1  1  ]
	%  y(:, end+1) = 3 results in
	%[
	%  1  1  1  3
	%  1  1  1  3
	%  1  1  1  3
	%]
    out(:, end + 1) = (X1 .^ (i - j)) .* (X2 .^ j);
   end
  end
end

%X1 =       X2=
%[a1       [b1   
% a2        b2
% a3]       b3]
% For degree = 2, out =
% [1   a1   b1   a1^2   a1*b1    b1^2 
%  1   a2   b2   a2^2   a2*b2    b2^2
%  1   a3   b3   a3^2   a3*b3    b3^3]

% For degree = 3, out =
% [1   a1   b1   a1^2   a1*b1    b1^2   a1^3   a1^2 * b1   a1 * b1^2   b1^3
%  1   a2   b2   a2^2   a2*b2    b2^2   a2^3   a2^2 * b2   a2 * b2^2   b2^3
%  1   a3   b3   a3^2   a3*b3    b3^3   a3^3   a3^2 * b2   a3 * b3^2   b3^3]